import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/message.dart';
import 'dart:io';
import 'package:http/http.dart' as http;

import 'common.dart';
import 'methods.dart';

class DataBase {
  static final lstCommonCollection = ['Account', 'Language'];
  static CollectionReference getCollection(String collectionName) {
    if (lstCommonCollection.contains(collectionName)) {
      return FirebaseFirestore.instance.collection(collectionName);
    }
    return FirebaseFirestore.instance
        .collection(Common.mainCollection)
        .doc(Common.mainDocument)
        .collection(collectionName);
  }

  static Query getQuery(String collectionName,
      {Map<String, Object> conditions,
      Map<String, bool> orderBy,
      int limit = -1}) {
    if (limit == null) {
      limit = -1;
    }

    Query collection = DataBase.getCollection(collectionName);
    try {
      if (conditions != null && conditions.length > 0) {
        conditions.forEach((key, value) {
          collection = collection.where(key, isEqualTo: value);
        });
      }

      if (orderBy != null && orderBy.length > 0) {
        orderBy.forEach((key, value) {
          collection = collection.orderBy(key, descending: value);
        });
      }
      if (limit > 0) {
        collection = collection.limit(limit);
      }
    } catch (e) {
      print('getCollection : ' + e.toString());
    }

    return collection;
  }

  static int getNewDocumentID() {
    String documentId = DateTime.now()
        .difference(DateTime.utc(2000, 1, 1))
        .inSeconds
        .toString();
    return Methods.stringToInt(documentId);
  }

  static Map getNewDataTemplate(Map<String, Object> dict, int uid) {
    dict["documentID"] = getNewDocumentID();
    dict["uid"] = uid;
    dict["statusData"] = 1;
    dict["createDate"] = FieldValue.serverTimestamp();

    return dict;
  }

  static Map getModified(Map<String, Object> dict, int uid) {
    dict["modifiedUid"] = uid;
    dict["modifiedDate"] = FieldValue.serverTimestamp();
    return dict;
  }

  static Future setData(
      Map<String, Object> data, String colllectionPath, int uid) async {
    bool isNew;
    if (data.containsKey('documentID')) {
      isNew = false;
      data = getModified(data, uid);
    } else {
      isNew = true;
      data = getNewDataTemplate(data, uid);
    }
    int documentID = Methods.stringToInt(data['documentID'].toString());
    if (isNew) {
      getCollection(colllectionPath)
          .doc(documentID.toString())
          .set(data, SetOptions(merge: true));
    } else {
      getCollection(colllectionPath).doc(documentID.toString()).update(data);
    }
  }
}
